package net.beaumaris.cachetest;

import java.util.Set;

import javax.transaction.Status;
import javax.transaction.TransactionManager;

import org.infinispan.Cache;
import org.infinispan.distexec.DistributedCallable;

public abstract class BaseTask<T> implements DistributedCallable<Integer, Integer, T> {

   private static final long serialVersionUID = 1L;

   protected Cache<Integer, Integer> allocCache;
   protected Cache<String, Integer> uuidCache;

   private TransactionManager tm;

   @Override
   public T call() throws Exception {
      try {

         tm.begin();

         return doTx();

      } catch (Exception e) {

         tm.setRollbackOnly();
         throw e;
      }

      finally {

         int status = -1;
         try {
            status = tm.getStatus();
         } catch (Exception e) {

         }

         if (status == Status.STATUS_ACTIVE) {
            tm.commit();
         } else {
            tm.rollback();
         }
      }
   }

   protected abstract T doTx();

   @Override
   public void setEnvironment(Cache<Integer, Integer> cache, Set<Integer> inputKeys) {

      this.allocCache = cache;
      this.uuidCache = cache.getCacheManager().getCache("uuid");
      this.tm = cache.getAdvancedCache().getTransactionManager();
   }
}
