package net.beaumaris.cachetest;

import java.io.IOException;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.Executors;
import java.util.concurrent.ThreadPoolExecutor;

import javax.transaction.TransactionManager;

import junit.framework.Assert;

import org.infinispan.Cache;
import org.infinispan.config.Configuration;
import org.infinispan.manager.DefaultCacheManager;
import org.junit.After;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

public class RehashTest extends BaseCacheTest {

   Configuration c;

   @BeforeClass
   public static void init() {
      System.setProperty("java.net.preferIPv4Stack", "true");
   }

   @Before
   public void setup() throws InterruptedException, IOException {
      this.createNodes();

      DefaultCacheManager cm = cacheContainers.get(0);
      c = cm.getDefaultConfiguration();

   }

   @After
   public void stop() throws Exception {

      // Try and acquire all locks
      for (int i = 1; i < KEY_UPPER_BOUND; i++) {

         int index = r.nextInt(cacheContainers.size());
         DefaultCacheManager cm = cacheContainers.get(index);
         Cache<Integer, Integer> allocCache = cm.getCache("allocCache");

         TransactionManager tm = allocCache.getAdvancedCache().getTransactionManager();

         try {

            tm.begin();

            allocCache.getAdvancedCache().lock(i);

            Integer value = allocCache.get(i);
            if (value == null)
               value = new Integer(0);

            allocCache.put(i, value);
            tm.commit();

         } catch (Exception e) {
            tm.rollback();
            throw e;
         }
      }

      this.shutdown();
   }

   @Test
   public void testRehashJoin() throws Exception {

      CountDownLatch latch = new CountDownLatch(NUM_NODES * NUM_THREADS);

      SharedStats stats = new SharedStats();
      ThreadPoolExecutor executor = (ThreadPoolExecutor) Executors.newFixedThreadPool(NUM_NODES * NUM_THREADS);

      for (DefaultCacheManager cm : cacheContainers) {

         for (int i = 0; i < NUM_THREADS; i++) {

            StressingThread thread = new StressingThread(stats, cm, latch);
            executor.execute(thread);
         }
      }

      // Wait for a few seconds.
      Thread.sleep(5000);

      System.out.println("Adding a node...");

      // Try and add a cache
      try {
         addNode();
      } catch (Exception e) {
         Assert.fail();
      }

      latch.await();
      executor.shutdown();

      System.out.println(stats);

   }
}
